from atek_main.connect import *
import pandas as pd
from pandas.tseries.offsets import DateOffset
import numpy as np
import cytoolz.curried as tz
from typing import List, Optional, Union
from fnmatch import fnmatch
from pathlib import Path
from datetime import date


def orders_by_client(refresh: bool=False) -> pd.DataFrame:
    path = root() / "data/Orders by Client.csv"
    sql = """
    select
        curdate() as As_of_Date
        ,orders.orderNo as Order_Number
        ,orders.clientID as Client_ID
        ,clients.Client
        ,orders.order_date as Order_DateTime
        ,orders.status as Order_Status
        ,orders.prop_state as Property_State
        ,assignment.appr_fee as Appraiser_Fee
        ,invoices.total as Client_Fee
        ,case
            when orders.status = 'Complete' then invoices.total - assignment.appr_fee
            else null
            end as Admin_Fee
    from orders
        left join clients on orders.clientID = clients.ID
        left join order_assignment assignment on orders.orderNo = assignment.orderno
        left join invoices on orders.orderNo = invoices.order_num
    where
        orders.order_date >= '2017-01-01'
    """
    if path.exists() and refresh:
        path.unlink()

    def read():
        return tz.pipe(
            pd.read_csv(path, parse_dates=["As_of_Date", "Order_DateTime"])
            .assign(**{
                "Order_Date": lambda df: df.Order_DateTime.dt.strftime("%Y-%m-%d").astype("datetime64"),
                "Order_Month": lambda df: df.Order_Date.dt.strftime("%Y-%m"),
                "Order_Year": lambda df: df.Order_Date.dt.year,
            })
            .query("Order_Status != 'Cancelled'")
        )

    if path.exists():
        return read()

    return tz.pipe(
        query(sql)
        .to_csv(path, index=False)
        ,lambda _: read()
    )


def report_orders_by_client(refresh: bool=False, pretty: bool=False, export: bool=False) -> pd.DataFrame:
    data = orders_by_client(refresh)
    path = root() / f"reports/Orders by Client {data.As_of_Date.max().strftime('%Y-%m_%d')}.xlsx"

    return tz.pipe(
        orders_by_client(refresh)
        .assign(**{
            "Rel_Year": lambda df: df.Order_Year - df.As_of_Date.dt.year,
            "YTD_Date": lambda df: (df.As_of_Date + (df.Rel_Year * np.timedelta64(1, "Y"))).dt.normalize(),
            "YTD": lambda df: np.where(df.Order_Date <= df.YTD_Date, 1, 0),
        })
        ,lambda df: pd.concat([
            df.query("YTD == 1").assign(**{"Period": "YTD"}),
            df.assign(**{"Period": "All"}),
        ])
        .groupby(["As_of_Date", "Client_ID", "Client", "Order_Year", "Period"])
        .agg(**{
            "Orders": ("Order_Number", "size")
        })
        .reset_index()
        ,lambda df: df.merge(
            df
                .assign(**{
                    "Prior_Orders": df.Orders,
                    "Prior_Year": df.Order_Year + 1,
                })
                .filter(["Client_ID", "Period", "Prior_Year", "Prior_Orders"]),
            left_on=["Client_ID", "Period", "Order_Year"],
            right_on=["Client_ID", "Period", "Prior_Year"],
            how="left"
        )
        .drop(columns=["Prior_Year"])
        .assign(**{
            "Prior_Orders": lambda df: df.Prior_Orders.fillna(0),
            "Client_Status": lambda df:
                np.where(df.eval("Prior_Orders > 0 and Orders == 0"), "Lost",
                np.where(df.eval("Prior_Orders > 0"), "Existing",
                "New"
            )),
            "Change_YoY": lambda df: df.Orders - df.Prior_Orders,
            "Pct_Change_YoY": lambda df: np.where(df.Prior_Orders > 0, df.Change_YoY / df.Prior_Orders, pd.NA),
            "Pct_Orders_by_Year": lambda df: df.Orders / df.groupby(["Period", "Order_Year"])["Orders"].transform("sum"),
            "Pct_Orders_by_Year_Client_Status": lambda df: df.Orders
                / df.groupby(["As_of_Date", "Period", "Order_Year", "Client_Status"])["Orders"].transform("sum"),
            "Rank_by_Year": lambda df: df.groupby(["Period", "Order_Year"])["Orders"]
                .rank(ascending=False, method="min"),
            "Rank_by_Client_Status": lambda df: df.groupby(["As_of_Date", "Period", "Order_Year", "Client_Status"])["Orders"]
                .rank(ascending=False, method="min"),
        })
        ,lambda df: df.rename(columns={col: col.replace("_", " ") for col in df.columns}) if pretty else df
        ,lambda df: df.to_excel(path, sheet_name="Orders by Client", index=False) if export else df
    )

#  report_orders_by_client(False, True, True)
import os
for k, v in os.environ.items():
    if "SSH_" in k or "DB_" in k:
        print(f"{k}:{v}")

tz.pipe(
    query("select * from orders order by order_date desc limit 10")
    ,vert
    ,show
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pandas as pd
from fnmatch import fnmatch
from tabulate import tabulate
from cytoolz.curried import pipe, curry, take, identity
from textwrap import fill
from typing import Union, Optional, List, Any

__all__ = ["is_df", "filter_col", "vert", "show"]

def is_df(obj: Any, param_name: str="dataframe") -> pd.DataFrame:
    if isinstance(obj, pd.DataFrame):
        return obj

    raise ValueError(f"arg '{param_name}' must be a pandas.DataFrame")


@curry
def filter_col(patterns: Union[str, List[str]],
    dataframe: pd.DataFrame) -> pd.DataFrame:
    """Filteres the columns of a dataframe using list
    of fnmatch patterns"""

    patterns = [patterns] if isinstance(patterns, str) else patterns
    columns = list(df.columns)
    filtered = set(
        col
        for col in is_df(dataframe).columns
        for pat in patterns
        if fnmatch(col.lower(), pat.lower())
    )
    return df.filter(list(filtered))


@curry
def vert(dataframe: pd.DataFrame, limit: Optional[int]=3) -> pd.DataFrame:
    """Transposes dataframe columns to rows"""

    return pipe(
        dataframe
        ,is_df
        ,lambda df: df.head(limit) if limit else df
        ,lambda df: df.transpose()
        .reset_index()
        ,lambda df: df.rename(columns={
            col:("column" if col == "index" else f"row {int(col)+1:02}")
            for col in df.columns
        })
    )


@curry
def show(dataframe: pd.DataFrame, limit: Optional[int]=None, 
    print_out: bool=True, col_width: int=20, **kwds) -> str:
    """Returns a DataFrame as an asci table using tabulate"""

    defaults = {
        "headers": "keys",
        "tablefmt": "fancy_grid",
    }

    args = {**defaults, **kwds}

    return pipe(
        dataframe
        ,is_df
        ,lambda df: df.head(limit) if limit else df
        ,lambda df: df.to_dict("records")
        ,lambda table: [
            {
                fill(str(k), col_width): fill(str(v), col_width)
                for k, v in row.items()
            }
            for row in table
        ]
        ,lambda x: tabulate(x, **args)
        ,lambda x: print(x) if print_out else x
    )



"""Provides basic utilities to query a mysql database over an ssh tunnel"""
from sshtunnel import SSHTunnelForwarder, open_tunnel
import pymysql
import pandas as pd
from pandas.tseries.offsets import DateOffset
import numpy as np
import cytoolz.curried as tz
from typing import List, Optional, Union
from fnmatch import fnmatch
from pathlib import Path
from datetime import date
from dotenv import load_dotenv
import os
from textwrap import fill
from tabulate import tabulate


__all__ = ["query", "root", "load_env"]


def root():
    return Path(__file__).parent

DOT_ENV_PATH = Path(root() / ".env").resolve()

def load_env(path: Optional[Path]=None):
    """Loads contents of {path} to environment variables"""
    global DOT_ENV_PATH
    if path:
        DOT_ENV_PATH = path

    if not DOT_ENV_PATH.exists():
        raise ValueError(f"DOT_ENV_PATH = '{DOT_ENV_PATH}' does not exist")

    result = load_dotenv(dotenv_path=DOT_ENV_PATH, override=True)

    if result:
        return f"load_env loaded '{DOT_ENV_PATH}'"
    else:
        return ValueError(f"load_env could not load '{DOT_ENV_PATH}'")

print(load_env())

@tz.curry
def query(sql: str) -> pd.DataFrame:
    """Returns a dataframe from provided {sql}"""
    ssh_host = os.getenv("SSH_HOST")
    ssh_port = int(os.getenv("SSH_PORT"))
    ssh_pass = os.getenv("SSH_PASS")
    ssh_user = os.getenv("SSH_USER")
    db_host = os.getenv("DB_HOST")
    db_name = os.getenv("DB_NAME")
    db_user = os.getenv("DB_USER")
    db_pass = os.getenv("DB_PASS")
    with SSHTunnelForwarder(
        (ssh_host, ssh_port),
        ssh_password=ssh_pass,
        ssh_username=ssh_user,
        remote_bind_address=(db_host, 3306)) as tunnel:

        conn = pymysql.connect(
            host=db_host,
            user=db_user,
            password=db_pass,
            port=tunnel.local_bind_port,
            db=db_name,
        )

        df = pd.read_sql(sql, conn)
        conn.close()
        return df


if __name__ == "__main__":
    for k, v in os.environ.items():
        if "SSH_" in k or "DB_" in k:
            print(f"{k}:{v}")

    from atek_main.df_toolz import show
    show(query("select version() as version"))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['atek_main']

package_data = \
{'': ['*'], 'atek_main': ['data/*', 'reports/*']}

install_requires = \
['altair>=4.1.0,<5.0.0',
 'cytoolz>=0.10.1,<0.11.0',
 'jupyter>=1.0.0,<2.0.0',
 'mysql-connector-python>=8.0.21,<9.0.0',
 'numpy>=1.19.1,<2.0.0',
 'openpyxl>=3.0.4,<4.0.0',
 'pandas>=1.0.5,<2.0.0',
 'paramiko>=2.7.1,<3.0.0',
 'pymysql>=0.10.0,<0.11.0',
 'python-dotenv>=0.14.0,<0.15.0',
 'sshtunnel>=0.1.5,<0.2.0',
 'tabulate>=0.8.7,<0.9.0']

setup_kwargs = {
    'name': 'atek-main',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'JediHero',
    'author_email': 'hansen.rusty@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
